
/*Unified Spline Macro*/

/*Graphics Setup for Spline Macros

The following code should be run before running any of the spline macros as it
sets up graphic styles, devices and default fonts, line styles and colors.

The GRAPHSET macro below should be run with your system settings.
*/

/*Create Graphics Template Defaults*/
/*You only have to ever run PROC TEMPLATE once on a system to create the style
  if you have write permission to where styles are stored.  Otherwise, once per session */
proc template;
define style Styles.splinecurve;
parent = Styles.Listing;
class GraphWalls /
      linethickness = 3px
      linestyle = 1
      frameborder = off
      contrastcolor = GraphColors('gaxis')
      backgroundcolor = white
      color = GraphColors('gwalls');
   class GraphAxisLines /
      tickdisplay = "outside"
      linethickness = 3px
      linestyle = 1
      contrastcolor = GraphColors('gaxis')
      color = GraphColors('gaxis'); 
class GraphFonts
      "Fonts used in graph styles" /
      'NodeDetailFont' = ("<sans-serif>, <MTsans-serif>",16pt)
      'NodeLinkLabelFont' = ("<sans-serif>, <MTsans-serif>",20pt)
      'NodeInputLabelFont' = ("<sans-serif>, <MTsans-serif>",20pt)
      'NodeLabelFont' = ("<sans-serif>, <MTsans-serif>",20pt)
      'NodeTitleFont' = ("<sans-serif>, <MTsans-serif>",20pt)
      'GraphDataFont' = ("<sans-serif>, <MTsans-serif>",16pt)
      'GraphUnicodeFont' = ("<MTsans-serif-unicode>",20pt)
      'GraphValueFont' = ("<sans-serif>, <MTsans-serif>",16pt)
      'GraphLabel2Font' = ("<sans-serif>, <MTsans-serif>",20pt)
      'GraphLabelFont' = ("<sans-serif>, <MTsans-serif>",20pt,bold)
      'GraphFootnoteFont' = ("<sans-serif>, <MTsans-serif>",10pt)
      'GraphTitleFont' = ("<sans-serif>, <MTsans-serif>",15pt,bold)
      'GraphTitle1Font' = ("<sans-serif>, <MTsans-serif>",20pt,bold)
      'GraphAnnoFont' = ("<sans-serif>, <MTsans-serif>",20pt);
class body /                                                            
         fontfamily = "Arial, sans-serif"                                          
         color = #000000                                                      
         backgroundcolor = #FFFFFF;  
class Header/
 background=white
 foreground=black;  
end; 
run; 

%macro graphset(gpath,dpi,style,format=emf,type=listing,);
/*GRAPHICS OUTPUT OPTIONS
Not all combinations of format and type are supported
TYPE            FORMATS
Listing         PNG (default), GIF, BMP, DIB, EMF, EPSI, GIF, JFIF, JPEG, PBM, PS, TIFF, WMF
HTML            PNG (default), GIF, JPEG,
LATEX           PostScript (default), EPSI, GIF, JPEG, PDF, PNG
PDF             PDF

Parameters:
gpath       Location where graphic output will be stored.  This is a filename for PDF output
            and a directory for all other output types.
dpi         Dots per inch.  Not relevant for PDF or vector graphics [EMF]
style       Which style to use.  "splinecurve" is recommended
format      Which graphic format to use.  Default is "EMF'.
type        Which output destination to use.  Default is "Listing".

Tested combinations include Listing/EMF, Listing/JPEG, HTML/JPEG and PDF/PDF.  Note that
PDF files can be converted to PS or EPS output by Acrobat Pro or Inkscape
ODS1-ODS3 are needed in a macro before output is produced.
ODS4 is needed at the end of the output, especially if PDF output is desired.
     to close the PDF file.
*/
%global ods1 ods2 ods3;

%if &style eq  %then %let style=default;
%if &dpi eq  %then %let dpi=300;
%let ods1= ods &type gpath="&gpath" style=&style
   image_dpi = &dpi; 

%let ods2= ods graphics on / width=10in height=6.8in IMAGEFMT=&format ;
run;
%if %upcase(&type) eq PDF %then %do;
%let ods1= ods pdf file="&gpath" style=&style;
%let ods3=options orientation=landscape nodate nonumber;
%end;
%mend graphset;

/*  Sample executions of %graphset:
%graphset(C:\Users\David\Documents\figure output\test1.pdf,,splinecurve,format=pdf,type=pdf);
%graphset(C:\Users\David\Documents\figure output,,splinecurve,format=emf,type=listing);
%graphset(C:\Users\gagnon\Documents\figure output,,splinecurve,format=jpeg,type=listing);
*/


/*Set default graphic attributes for spline macros*/
data graphattr;
/*scatter and series use linecolor, band uses fillcolor*/
length ID $ 7  linecolor $ 11 fillcolor $ 11  ;
input value  linecolor  fillcolor linethickness ;
markersymbol="circlefilled";
symbol = "circlefilled";
markersize=1;
filltransparency = .40;
linepattern = 'Solid';
if _n_ le 12 then ID = 'colors';
else ID='grays';
datalines;
 1  black       black      2
 2  red         red        2
 3  blue        blue       2 
 4  green       green      2 
 5  cxD17800    cxD17800   2
 6  cxB26084    cxB26084   2
 7  cx2597FA    cx2597FA   2
 8  brown       brown      2
 9  yellow      yellow     2
 10 gray        gray       2
 11 lightred    lightred   2
 12 lightgreen  lightgreen 2
 1  black       black      2
 2  gray        grayA8     3
 3  black       gray48     5
 4  gray        gray       5
;
run;

options mprint symbolgen mlogic spool ;


/***********************************************************************************
*                                                                                  *
*   Spline macro for non-survival outcomes without effect modification/subgroups   *
*                                                                                  *
***********************************************************************************/
%macro glimsp(data,var,refval,covar,class,outcome,nknots,method,dist,
               link,offset,weight,min_x,max_x,by_x,min_y,max_y,by_y,bins);
/*
  data                      Data set name;
  var                       Splined predictor
  refval                    Reference value of spline variable
  covar                     Model covariates
  class                     Class statement variables
  outcome                   Outcome variable

  nknots                    Number of internal knots
  method                    Knot placement method:  1=Even knot placement  
                            2= Percentile based   3= Harrell's approach
  dist                      Distribution
  link                      Link function ["identity","logit" or "log"]
  offset                    Offset variable if needed.  "NONE" or missing value eliminates it.
  weight                    Weight variable, if needed.  May also be missing or "NONE"
  min_x, max_x by_x         minimum, maximum and interval values on X axis of plot
  min_y, max_y by_y         minimum, maximum and interval values of Y axis of plot
  bins                      Number of bars in the final histogram


  Note that a predicted risk value for the reference value is calculated and appears at the 
  end of the output. For models with an offset, the predicted value will be calculated for 
  offset=0, so units matter. If you want annual rates, make sure that the offset unit is in 
  log(years).
  For binary distribution models, the code assumes that events are coded as 1 for an event
  and zero for a non-event, modeling the P(event=1).

*/

** Calculated;
      %let var = %upcase(&var);
      %let link = %upcase(&link);
      %if &link = ID %then %let link = IDENTITY;



options ps = 1000 validvarname=V7 symbolgen mlogic;

/*Calculate Harrell's Knot Placement*/
data FHps;
array pcts(&nknots) pcts1-pcts&nknots;
  expt = ceil(&nknots/3)-1;
  pcts(1) = 0.1/2**expt;
  pcts(&nknots) = 1-pcts(1);
  range = 1-2*pcts(1);
   incr = range/(&nknots-1);
  do i = 2 to &nknots-1;
     pcts(i) = pcts(i-1)+incr;
   end;
  do i = 1 to &nknots;
    pcts(i) = 100*round(pcts(i),.0001);
  end;
   keep pcts1-pcts&nknots;
   run;
   proc print data=FHps;
   title 'Harrell knot locations';
   run;
data _null_;
set FHps;
call symput('FHpcts',catx(' ', of _numeric_));
run;
proc univariate data=&data noprint;
title;
   var &var;
   output out=percentiles pctlpts=&FHpcts pctlpre=p_;run;
data _null_;
set percentiles;
call symput('FHcuts',catx(',', of _numeric_));
run;

/*Create plot points [default is 100 values from min to max of &var variable]*/
/*Prepare data set for spline estimation and graphing*/
proc means data=&data;
title;
var &var  ;
output out=meanout min=minx minem max=maxx maxem;
run;

data newdata; set meanout;
interval = (maxx-minx)/100;
        do &var = minx to maxx by interval; 
          output;
        end; 
        keep &var ;
        run;
proc sql noprint;
select &var into :cat1-:cat101 from newdata;
run;
quit;



/*Spline modeling:*/
ods output FitStatistics=Fitspline;
proc glimmix data=&data method=laplace empirical;
  class &class;
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
model &outcome (event='1') = spl &covar /  solution dist=&dist link=&link
%if (&offset ne ) & (&offset ne NONE) %then %do; offset = &offset %end;;
  output out=tmp1 ;
  store preddat;
run;
quit;
data Fitspline; set Fitspline;
linenum=_n_;
rename Value=Spline;
run;

/*Linear modeling:*/
ods output FitStatistics=FitLine;
proc glimmix data=&data method=laplace empirical;
  class &class;
    %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
model &outcome (event='1')= &var &covar /  solution dist=&dist link=&link
%if (&offset ne ) & (&offset ne NONE)  %then %do; offset = &offset %end;;
run;
quit;
data FitLine; set FitLine;
linenum=_n_;
rename Value=Line;
run;
/*Quadratic modeling:*/
ods output FitStatistics=FitQuad;
proc glimmix data=&data method=laplace empirical;
  class &class;
    %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
model &outcome (event='1')= &var &var * &var &covar /  solution dist=&dist link=&link
%if (&offset ne ) & (&offset ne NONE)  %then %do; offset = &offset %end;;
run;
quit;
data FitQuad; set FitQuad;
linenum=_n_;
rename Value=Quadratic;
run;
data FitStats ; merge Fitspline FitLine FitQuad end=eof; by linenum;
retain m2SLL m2LLL m2QLL DF_SLL DF_LLL DF_QLL p_SvL p_SvQ; 
LN=_n_;
if _n_ eq 1 then do;
   m2SLL = Spline;
   m2LLL = Line;
   m2QLL = Quadratic;
end;
if _n_ eq 2 then do;
   DF_SLL = (Spline - m2SLL)/2;
   DF_LLL = (Line - m2LLL)/2;
   DF_QLL = (Quadratic - m2QLL)/2;
end;
output;
if eof then do;
   DeltaSL = abs(m2SLL - m2LLL);
   DeltaSQ = abs(m2SLL - m2QLL);
   p_SvL = 1-cdf('CHISQUARE',DeltaSL,(DF_SLL-DF_LLL));
   p_SvQ = 1-cdf('CHISQUARE',DeltaSQ,(DF_SLL-DF_QLL));
LN=9; Descr='Model DFs';Spline=DF_SLL; Quadratic=DF_QLL; Line=DF_LLL;
output;
LN=10; Descr='Chi-square vs. Spline'; Spline=.; Quadratic=DeltaSQ; Line=DeltaSL;
output;
LN=11; Descr='p-value vs. Spline'; Spline=.; Quadratic=p_SvQ; Line=p_SvL;
output;
end;
format Spline Line Quadratic pvalue12.4;
keep Descr Spline  Quadratic Line;
run;


proc plm restore=preddat ;
title 'Spline knot locations';
show effects;
run;
quit;
ods listing close;
ods output Estimates=plmout;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   estimate "&&cat&numcat"  
            spl [-1, &refval] [1, &&cat&numcat] /exp cl;
%end;

run;
quit;
proc print data=plmout;
title;
run;
ods listing;
data plmout2; set plmout;
&var = Label/1;
if UpperExp gt &max_y then UpperExp = &max_y;
if LowerExp lt &min_y then LowerExp = &min_y;
run;
ods listing;
/*CRUDE model for estimates*/
proc glimmix data=&data method=laplace empirical;
%if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
model &outcome (event='1')= spl /  solution dist=&dist link=&link
%if (&offset ne ) & (&offset ne NONE)  %then %do; offset = &offset %end;;
  store preddat2;
run;
quit;

data newdata2; set meanout;
interval = (maxx-minx)/100;
&offset = 0;
        do &var = minx to maxx by interval;
             output;
        end; 
        keep &var &offset;
        run;
proc plm restore=preddat2 ;
show effects;

score data=newdata2 out=crudeout predicted=pred lclm=lowerp uclm=upperp lcl=lowers ucl=uppers / ilink;
run;
quit;



/*/*Adjusted model for estimates*/*/
/*proc glimmix data=&data method=laplace empirical;*/
/*class &class;*/
/*  %if (&weight ne ) & (&weight ne NONE) %then %do;/* weight &weight %end;;*/
/*%if &method = 1 %then %do;*/
/*effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;*/
/*%if &method = 2 %then %do;*/
/*effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;*/
/*%if &method = 3 %then %do;*/
/*effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;*/
/*model &outcome (event='1')= spl &covar/  solution dist=&dist link=&link*/
/*%if (&offset ne ) & (&offset ne NONE)  %then %do; offset = &offset %end;;*/
/*  store preddat3;*/
/*run;*/
/*quit;*/
/**/
/**/
/*proc plm restore=preddat3 ;*/
/*show effects;*/
/**/
/*score data=newdata2 out=adjout predicted=pred lclm=lowerp uclm=upperp /ilink;*/
/*run;*/
/*quit;*/



/*Graph it*/

options orientation=landscape;
&ods1; &ods2; &ods3;

/********************************* For non-Gaussian Outcomes ***********************************/
%if &link ne IDENTITY %then %do;
ods graphics / imagename="Odds-Risk Ratios";

proc sgplot data=plmout2 noautolegend dattrmap=graphattr;
title ;
series y=ExpEstimate x=&var / lineattrs=(thickness=3 color=black);
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  values=(&min_y to &max_y by &by_y) label=
%if &link=LOGIT %then %do;"Odds Ratio" %end;
%if &link=LOG %then %do;"Risk Ratio" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;
;
band x= &var
 lower=LowerExp upper=UpperExp / fillattrs=(color=gray transparency=0.6) ;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;
ods graphics / imagename="Odds-Risk LogX";

proc sgplot data=plmout2 noautolegend dattrmap=graphattr;
title "Log_e(&var)";;
series y=ExpEstimate x=&var / lineattrs=(thickness=3 color=black);
xaxis  TYPE=LOG LOGSTYLE=LOGEXPAND LOGBASE=e values=(&min_x to &max_x by &by_x);
yaxis  values=(&min_y to &max_y by &by_y) label=
%if &link=LOGIT %then %do;"Odds Ratio" %end;
%if &link=LOG %then %do;"Risk Ratio" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;;
band x= &var
 lower=LowerExp upper=UpperExp / fillattrs=(color=gray transparency=0.6) ;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;


data crudeout; set crudeout;
if &min_x lt &var lt &max_x;
run;
ods graphics / imagename="Crude Predicted Prob-Rate";

proc sgplot data=crudeout noautolegend dattrmap=graphattr;
title ;
series y=pred x=&var / lineattrs=(thickness=3 color=black);
yaxis  label=
%if &link=LOGIT %then %do;"Predicted Probability" %end;
%if &link=LOG %then %do;"Predicted Rate" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;
;
band x= &var
 lower=lowerp upper=upperp / fillattrs=(color=gray transparency=0.6);
run;
quit;
%end;
/********************************************For Gaussian Outcomes *************************/
%else %do;
/*Predicted value plot -- includes IDENTITY link*/
data crudeout; set crudeout;
if &min_x lt &var lt &max_x;
run;
ods graphics / imagename="Predicted-Mean";

proc sgplot data=crudeout noautolegend dattrmap=graphattr;
title ;
series y=pred x=&var / lineattrs=(thickness=3 color=black);
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  label= "&outcome" ;
band x= &var
 lower=lowerp upper=upperp / fillattrs=(color=gray transparency=0.6) ;
run;
quit;

ods graphics / imagename="Predicted-Subject";

proc sgplot data=crudeout noautolegend dattrmap=graphattr;
title ;
series y=pred x=&var / lineattrs=(thickness=3 color=black);
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  label= "&outcome";
band x= &var
 lower=lowers upper=uppers / fillattrs=(color=gray transparency=0.6) ;
run;
quit;

ods graphics / imagename="Scatterplot"  ;
proc sgplot data=&data dattrmap=graphattr;
scatter x=&var y=&outcome / markerattrs = (symbol = circlefilled color = black size=4) 
  transparency=0.5;
xaxis  values=(&min_x to &max_x by &by_x)  ;
run;
quit;

%end;

ods graphics / imagename="Histogram";

proc sgplot data=&data;
Title;
/*where &min_x le &var le &max_x;*/
histogram &var / nbins=&bins scale=count fillattrs=(color=gray transparency=0.7);
run;

options ls=80 ps=55;

ods listing;
proc print data=FitStats;
title'Fit statistics and LRT vs. Spline';
run;


ods pdf close;

%mend glimsp;
run;
/*
Sample execution:
%glimsp(data=analytic,var=age,refval=130,covar=sex,class=sex,outcome=sysbp,nknots=5,method=3,dist=NORMAL,
          link=IDENTITY,offset=,weight=NONE,min_x=30,max_x=70,by_x=10,min_y=80,max_y=300,
          by_y=20, bins=30);
run;

*/


/***********************************************************************************
*                                                                                  *
*   Spline macro for non-survival outcomes with effect modification/subgroups      *
*                                                                                  *
***********************************************************************************/

%macro glimspem(data,var,refval,emvar,emref,covar,class,outcome,nknots,method,
       dist,link,offset,weight,min_x,max_x,by_x,min_y,max_y,by_y,CI=YES,color=YES,bins=20);

/*
  data                      Data set name;
  var                       Splined predictor
  refval                    Reference value of spline variable
  emvar                     Stratifying/Effect modifier variable [numeric, coded 1-N]
  emref                     Reference category for effect modifier variable
  covar                     Model covariates
  class                     Class statement variables
  outcome                   Outcome variable
  nknots                    Number of internal knots
  method                    Knot placement method:  1=Even knot placement  
                            2= Percentile based   3= Harrell's approach
  dist                      Distribution
  link                      Link function ["identity","logit" or "log"]
  offset                    Offset variable if needed.  May be left blank, or use "NONE".
  weight                    Weight variable if needed.  May be left blank, or use "NONE".
  min_x, max_x by_x         minimum, maximum and interval values on X axis of plot
  min_y, max_y by_y         minimum, maximum and interval values of Y axis of plot
  CI                        Do you want confidence intervals?  Default is YES
  color                     Do you want color plots?  Default is YES, otherwise grayscale
  bins                      Number of bars in the histogram.  Default is 20;

  Note that a predicted risk value for the reference value is calculated and appears at the 
  end of the output. For models with an offset, the predicted value will be calculated for 
  offset=0, so units matter. If you want annual rates, make sure that the offset unit is in 
  log(years).
  For binary distribution models, the code assumes that events are coded as 1 for an event
  and zero for a non-event, modeling the P(event=1).
*/

%let link = %upcase(&link);

%if &color=YES %then %let attrid = colors;
%else %let attrid = grays;

options ps = 1000 validvarname=v7;

/*Calculate Harrell's Knot Placement*/
data FHps;
array pcts(&nknots) pcts1-pcts&nknots;
  expt = ceil(&nknots/3)-1;
  pcts(1) = 0.1/2**expt;
  pcts(&nknots) = 1-pcts(1);
  range = 1-2*pcts(1);
   incr = range/(&nknots-1);
  do i = 2 to &nknots-1;
     pcts(i) = pcts(i-1)+incr;
   end;
  do i = 1 to &nknots;
    pcts(i) = 100*round(pcts(i),.0001);
  end;
   keep pcts1-pcts&nknots;
   proc print data=FHps;
   run;
data _null_;
set FHps;
call symput('FHpcts',catx(' ', of _numeric_));
run;
proc univariate data=&data noprint;
   var &var;
   output out=percentiles pctlpts=&FHpcts pctlpre=p_;run;
data _null_;
set percentiles;
call symput('FHcuts',catx(',', of _numeric_));
run;

/*Prepare data set for spline estimation and graphing*/
proc means data=&data;
var &var  ;
output out=meanout min=minx  max=maxx ;
run;

data newdata; set meanout;
interval = (maxx-minx)/100;
        do &var = minx to maxx by interval; 
          output;
        end; 
        keep &var ;
        run;
proc sql noprint;
select &var into :cat1-:cat101 from newdata;
run;
quit;

ods output OneWayFreqs=owf;
proc freq data=&data; ****### - edit from data=analytic;
tables &emvar;
run;
proc print data=owf;
run;

data owf; set owf end=eof;
vname = cat('emvar',_n_);
call symput(vname,&emvar);
if eof then call symput("numcats",_n_);
run;
/*Spline modeling:*/
proc glimmix data=&data method=laplace empirical;
  class &emvar &class;
    %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
model &outcome (event='1') = &emvar spl &emvar*spl &covar /  solution dist=&dist link=&link
%if (&offset ne ) & (&offset ne NONE) %then %do; offset = &offset %end;;
  output out=tmp1 ;
  store preddat;
run;
quit;
ods listing;
proc plm restore=preddat ;
title 'Spline knot locations';
show effects;
run;
quit;
ods listing close;
ods output Estimates=plmout;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   %do emcat = 1 %to &numcats;
   estimate "&&emvar&emcat &&cat&numcat"  spl [1, &&cat&numcat] [-1, &refval]
            &emvar [1, &&emvar&emcat] [-1 &emref]
            &emvar*spl [1, &&emvar&emcat &&cat&numcat ] [-1, &emref &refval]  /ilink cl;
   %end;
%end;
run;
quit;
ods listing;
data plmout2; set plmout;
&emvar = scan(Label,1,' ')/1;
&var = scan(Label,2,' ')/1;
if Upper gt &max_y then Upper = &max_y;
if Lower lt &min_y then Lower = &min_y;
run;
ods listing;

/*Within EM value contrasts*/

ods listing close;
ods output Estimates=plmoutX;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   %do emcat = 1 %to &numcats;
   estimate "&&emvar&emcat &&cat&numcat"  spl [1, &&cat&numcat] [-1, &refval]
            &emvar [1, &&emvar&emcat] [-1 &&emvar&emcat]
            &emvar*spl [1, &&emvar&emcat &&cat&numcat ] [-1, &&emvar&emcat &refval]  /ilink cl;
   %end;
%end;
run;
quit;


data plmoutX2; set plmoutX;
&emvar = scan(Label,1,' ')/1;
&var = scan(Label,2,' ')/1;
if Upper gt &max_y then Upper = &max_y;
if Lower lt &min_y then Lower = &min_y;
run;


/*Time/range-based comparison*/
ods listing close;
ods output Estimates=tplmout;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   %do emcat = 1 %to &numcats;
   estimate "&&emvar&emcat &&cat&numcat"  spl [1, &&cat&numcat] [-1, &&cat&numcat]
            &emvar [1, &&emvar&emcat] [-1 &emref]
            &emvar*spl [1, &&emvar&emcat &&cat&numcat ] [-1, &emref &&cat&numcat]  /ilink cl;
   %end;
%end;
run;
quit;
ods listing;
data tplmout2; set tplmout;
&emvar = scan(Label,1,' ')/1;
&var = scan(Label,2,' ')/1;
if Upper gt &max_y then Upper = &max_y;
if Lower lt &min_y then Lower = &min_y;
run;
ods listing;
/*CRUDE model for estimates*/
proc glimmix data=&data method=laplace;
  class &emvar;
    %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
model &outcome (event='1') = &emvar spl &emvar*spl /  solution dist=&dist link=&link
%if (&offset ne ) & (&offset ne NONE)  %then %do; offset = &offset %end;;
  output out=tmp1 ;
  store preddat2;
run;
quit;

data newdata2; set meanout;
interval = (maxx-minx)/100;
%if (&offset ne ) & (&offset ne NONE)  %then %do; offset = 0; %end;;
        do &var = minx to maxx by interval;
          do &emvar = 1 to &numcats; 
             output;
          end;
        end; 
        keep &var &emvar &offset;
        run;
ods listing close;
proc plm restore=preddat2 ;
show effects;
score data=newdata2 out=crudeout predicted=pred 
   lclm=lowerp uclm=upperp lcl=lowers ucl=uppers/ilink;
run;
quit;

/*Graph it*/
options orientation=landscape;
&ods1; &ods2; &ods3;

/*Risk/Odds ratio plot*******************************************************************/
%if &link ne IDENTITY %then %do;
ods graphics / imagename="OR RR";

proc sgplot data=plmout2 noautolegend dattrmap=graphattr;
title ;
series y=Estimate x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  values=(&min_y to &max_y by &by_y) label=
%if &link=LOGIT %then %do;"Odds Ratio" %end;
%if &link=LOG %then %do;"Risk Ratio" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;
;
%if &CI eq YES %then %do;
band x= &var
 lower=Lower upper=Upper / group=&emvar attrid=&attrid ;
%end;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

/*Risk/Odds ratio plot with Log X axis*/
ods graphics / imagename="RR OR LogX";

proc sgplot data=plmout2 noautolegend dattrmap=graphattr;
title "Log_e(&var)";
series y=Estimate x=&var / group=&emvar attrid=&attrid;
xaxis  TYPE=LOG LOGSTYLE=LOGEXPAND LOGBASE=e values=(&min_x to &max_x by &by_x);
yaxis  values=(&min_y to &max_y by &by_y) label=
%if &link=LOGIT %then %do;"Odds Ratio" %end;
%if &link=LOG %then %do;"Risk Ratio" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;
;
%if &CI ne NO %then %do;
band x= &var
 lower=Lower upper=Upper / group=&emvar attrid=&attrid ;
%end;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

/*Risk/Odds ratio over X or time plot*/
ods graphics / imagename="RR OR over X";

proc sgplot data=tplmout2 noautolegend dattrmap=graphattr;
title ;
series y=Estimate x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  values=(&min_y to &max_y by &by_y) label=
%if &link=LOGIT %then %do;"Odds Ratio" %end;
%if &link=LOG %then %do;"Risk Ratio" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;
;
%if &CI eq YES %then %do;
band x= &var
 lower=Lower upper=Upper / group=&emvar attrid=&attrid ;
%end;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
run;
quit;

/*Within EM value contrasts*/
ods graphics / imagename="Within EM";

proc sgplot data=plmoutX2 noautolegend dattrmap=graphattr;
title ;
series y=Estimate x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis   label=
%if &link=LOGIT %then %do;"Odds Ratio" %end;
%if &link=LOG %then %do;"Risk Ratio" %end;
%if &link=IDENTITY %then %do; "&outcome" %end;
;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

%end;
/*Models involving IDENTITY link***********************************************************/
%else %do;
/*Predicted value plot -- includes IDENTITY link*/
data crudeout; set crudeout;
if &min_x lt &var lt &max_x;
run;
ods graphics / imagename="Predicted-Mean";

proc sgplot data=crudeout noautolegend dattrmap=graphattr;
title ;
series y=pred x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  label= "&outcome" ;
%if &CI ne NO %then %do;
band x= &var
 lower=lowerp upper=upperp / group=&emvar attrid=&attrid transparency=0.8 ;
%end;
/*refline 1 /axis=y lineattrs=(thickness=2 color=black);*/
/*refline &refval /axis=x lineattrs=(thickness=2 color=black);*/
run;
quit;
ods graphics / imagename="Predicted-Subject";

proc sgplot data=crudeout noautolegend dattrmap=graphattr;
title ;
series y=pred x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  label= "&outcome";
%if &CI ne NO %then %do;
band x= &var
 lower=lowers upper=uppers / group=&emvar attrid=&attrid transparency=0.8 ;
%end;
ods graphics / imagename="SGPANEL Scatterplot"  ;

proc sgpanel data=&data dattrmap=graphattr;
panelby &emvar;
scatter x=&var y=&outcome / markerattrs = (symbol = circlefilled color = black size=4) 
  transparency=0.5;
colaxis  values=(&min_x to &max_x by &by_x)  ;
run;
quit;

proc sort data=&data; by &emvar;

ods graphics / imagename="SGPLOT by Scatterplot"  ;
proc sgplot data=&data dattrmap=graphattr;
by &emvar;
scatter x=&var y=&outcome / markerattrs = (symbol = circlefilled color = black size=4) 
  transparency=0.5;
xaxis  values=(&min_x to &max_x by &by_x)  ;
run;
quit;

ods graphics / imagename="SGPLOT same Scatterplot"  ;
proc sgplot data=&data dattrmap=graphattr;
scatter x=&var y=&outcome / group=&emvar attrid=&attrid 
              markerattrs = (symbol = circlefilled size=4) transparency=0.5;
xaxis  values=(&min_x to &max_x by &by_x)  ;
run;
quit;
%end;
run;

ods graphics / imagename="GEM histogram";

proc sgpanel data=&data dattrmap=graphattr;
panelby &emvar;
histogram &var / nbins=&bins scale=count fillattrs=(color=gray transparency=0.7) ;
colaxis  values=(&min_x to &max_x by &by_x)  ;

run;

ods pdf close;

%mend glimspem;
/* Sample execution
%glimspem(data=analytic,var=BMI,refval=25,emvar=agegrp3,emref=1,
          covar=sex,class=sex,outcome=death,nknots=5,method=3,dist=poisson,
          link=log,offset=linkf,weight=NONE,min_x=15,max_x=40,by_x=5,min_y=0,max_y=10,
          by_y=2,CI=YES,color=YES);
run;
*/

/***********************************************************************************
*                                                                                  *
*   Spline macro for survival outcomes without effect modification/subgroups       *
*                                                                                  *
***********************************************************************************/

%macro phregsp(data,var,refval,covar,class,outcome,time0,time1,reftime,nknots,method,weight,
min_x,max_x,by_x,min_y,max_y,by_y);
/*
  data                      Data set name;
  var                       Splined predictor
  refval                    Reference value of spline variable
  covar                     Model covariates
  class                     Class statement variables.
  outcome                   Censoring indicator variable [0=censored]
  time0                     Study entry time.  If all subjects enter at zero, leave blank.
  time1                     Exit time variable.  If all subjects enter at zero, 
                            this is survival time
  reftime                   Reference time for estimates 
  nknots                    Number of internal knots
  method                    Knot placement method:  1=Even knot placement 
  weight                    Weight variable, if needed. May also be missing or "NONE".  
                            2= Percentile based   3= Harrell's approach
  min_x, max_x by_x         minimum, maximum and interval values on X axis of plot
  min_y, max_y by_y         minimum, maximum and interval values of Y axis of plot

*/
** Calculated;
      %let var = %upcase(&var);
options ps = 1000 validvarname=V7;


/*Calculate Harrell's Knot Placement*/
data FHps;
array pcts(&nknots) pcts1-pcts&nknots;
  expt = ceil(&nknots/3)-1;
  pcts(1) = 0.1/2**expt;
  pcts(&nknots) = 1-pcts(1);
  range = 1-2*pcts(1);
   incr = range/(&nknots-1);
  do i = 2 to &nknots-1;
     pcts(i) = pcts(i-1)+incr;
   end;
  do i = 1 to &nknots;
    pcts(i) = 100*round(pcts(i),.0001);
  end;
   keep pcts1-pcts&nknots;
   proc print data=FHps;
   run;
data _null_;
set FHps;
call symput('FHpcts',catx(' ', of _numeric_));
run;
proc univariate data=&data noprint;
   var &var;
   output out=percentiles pctlpts=&FHpcts pctlpre=p_;run;
data _null_;
set percentiles;
call symput('FHcuts',catx(',', of _numeric_));
run;


/*Create plot points [default is 100 values from min to max of &var variable]*/
/*Prepare data set for spline estimation and graphing*/
proc means data=&data;
var &var  ;
output out=meanout min=minx minem max=maxx maxem;
run;

data newdata; set meanout;
/*if _n_ then set tmpxk;*/
interval = (maxx-minx)/100;
        do &var = minx to maxx by interval; 
          output;
        end; 
        keep &var ;
        run;
proc sql noprint;
select &var into :cat1-:cat101 from newdata;
run;
quit;

/*Spline modeling:*/
/*Crude Predicted Survival for Reference Value Subject*/

proc phreg data=&data ;
title 'Crude Model';
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
%if &time0 eq %then %do;
model &time1*&outcome.(0) = spl;
%end;
%else %do;
model (&time0,&time1)*&outcome.(0) = spl;
%end;
baseline out=outtime survival=survival /diradj ;
store preddat2;
run;
quit;

data outtime; set outtime;
CUMincidence = (1-survival);
data outtime2; set outtime;
if &time1 le &reftime;
data outtime2; set outtime2 end=eof;
if eof;

proc print data=outtime2;
title "Predicted Crude Cumulative Incidence";
run;
data newdata2; set meanout;
interval = (maxx-minx)/100;
        do &var = minx to maxx by interval;
             output;
        end; 
        keep &var ;
        run;
proc plm restore=preddat2 ;
title 'Spline knot locations';
show effects;
run;
quit;




/*Run PHREG with spline variables and covariates*/
ods output FitStatistics=Fitspline;
proc phreg data=&data outest=pecov covout ;
title 'Multivariate Adjusted Spline Model';
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
class &class; 
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &time0 eq %then %do;
model &time1*&outcome.(0) = spl &covar;
%end;
%else %do;
model (&time0,&time1)*&outcome.(0) = spl &covar;
%end;
baseline out=outtimem survival=survival /diradj;
store preddat;
run;
quit;

data outtimem; set outtimem;
CUMincidence = (1-survival);
data outtime2m; set outtimem;
if &time1 le &reftime;
data outtime2m; set outtime2m end=eof;
if eof;

proc print data=outtime2m;
title "Predicted Multivariate Adjusted Cumulative Incidence for &var = &refval";
run;



proc print data=Fitspline;
run;
proc contents data=Fitspline;
run;
data Fitspline; set Fitspline;
linenum=_n_;
rename WithCovariates=Spline;
run;

ods output FitStatistics=FitQuad;
proc phreg data=&data outest=pecov covout ;
title 'Multivariate Adjusted Quadratic Model';
class &class; 
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &time0 eq %then %do;
model &time1*&outcome.(0) = &var &var * &var &covar;
%end;
%else %do;
model (&time0,&time1)*&outcome.(0) = &var &var * &var &covar;
%end;
run;
quit;
data FitQuad; set FitQuad;
linenum=_n_;
rename WithCovariates=Quadratic;
run;

ods output FitStatistics=FitLine;
proc phreg data=&data outest=pecov covout ;
title 'Multivariate Adjusted Linear Model';
class &class;
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &time0 eq %then %do;
model &time1*&outcome.(0) = &var &covar;
%end;
%else %do;
model (&time0,&time1)*&outcome.(0) = &var &covar;
%end;
run;
quit;

data FitLine; set FitLine;
linenum=_n_;
rename WithCovariates=Line;
run;

data FitStats ; merge Fitspline FitLine FitQuad end=eof; by linenum;
retain m2SLL m2LLL m2QLL DF_SLL DF_LLL DF_QLL p_SvL p_SvQ; 
length Cname $21.;
Cname=Criterion;
LN=_n_;
if _n_ eq 1 then do;
   m2SLL = Spline;
   m2LLL = Line;
   m2QLL = Quadratic;
end;
if _n_ eq 2 then do;
   DF_SLL = (Spline - m2SLL)/2;
   DF_LLL = (Line - m2LLL)/2;
   DF_QLL = (Quadratic - m2QLL)/2;
end;
output;
if eof then do;
   DeltaSL = abs(m2SLL - m2LLL);
   DeltaSQ = abs(m2SLL - m2QLL);
   p_SvL = 1-cdf('CHISQUARE',DeltaSL,(DF_SLL-DF_LLL));
   p_SvQ = 1-cdf('CHISQUARE',DeltaSQ,(DF_SLL-DF_QLL));
LN=4; Cname='Model DFs';Spline=DF_SLL; Quadratic=DF_QLL; Line=DF_LLL;
output;
LN=5; Cname='Chi-square vs. Spline'; Spline=.; Quadratic=DeltaSQ; Line=DeltaSL;
output;
LN=6; Cname='p-value vs. Spline'; Spline=.; Quadratic=p_SvQ; Line=p_SvL;
output;
end;
format Spline Line Quadratic pvalue12.4;
keep Cname Spline  Quadratic Line;
rename Cname=Criterion;
run;

options ls=80 ps=55;

ods listing;
proc print data=FitStats;
title'Fit statistics and LRT vs. Spline';
var Criterion Spline Quadratic Line;
run;

ods listing close;
ods output Estimates=plmout;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   estimate "&&cat&numcat"  
            spl [-1, &refval] [1, &&cat&numcat] /exp cl;
%end;

run;
quit;
ods listing;

data plmout2; set plmout;
&var = Label/1;
if UpperExp gt &max_y then UpperExp = &max_y;
if LowerExp lt &min_y then LowerExp = &min_y;
run;

/*Graph it*/

/*Calculate maximum cumulative incidence for MV adjusted and crude*/
proc means data=outtime noprint;
var CUMincidence;
output out=CI max=maxCI;
run;
proc means data=outtimem noprint;
var CUMincidence;
output out=CIm max=maxCIm;
run;
data _null_; merge CIm CI;
maxy = max(maxCI,maxCIm);
call symput('maxy',maxy);
run;


&ods1; &ods2; &ods3;
options orientation=landscape;
ods graphics / imagename="Crude CumIncidence";

proc sgplot data=outtime ;
title "Crude Cumulative Incidence Plot";
step x=&time1 y=CUMincidence / lineattrs=(thickness=2 color=black);
yaxis  label='Cumulative Incidence (%)' max=&maxy;
run;
quit;
ods graphics / imagename="MVACumIncidence";

proc sgplot data=outtimem ;
title "MV Adjusted Cumulative Incidence Plot";
step x=&time1 y=CUMincidence / lineattrs=(thickness=2 color=black);
yaxis  label='Cumulative Incidence (%)' max=&maxy ;
run;
quit;

ods graphics / imagename="STD Hazard Ratio";

proc sgplot data=plmout2 noautolegend ;
title ;
series y=ExpEstimate x=&var /  lineattrs=(thickness=3 color=black);
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  values=(&min_y to &max_y by &by_y) label= 'Hazard Ratio';
band x= &var
 lower=LowerExp upper=UpperExp / fillattrs=(color=gray transparency=0.6) ;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;

ods graphics / imagename="Hazard Ratio logX";

proc sgplot data=plmout2 noautolegend ;
where &min_x le &var le &max_x;
title "Log_e(&var)";;
series y=ExpEstimate x=&var /  lineattrs=(thickness=3 color=black);
xaxis  TYPE=LOG LOGSTYLE=LOGEXPAND LOGBASE=e values=(&min_x to &max_x by &by_x);
yaxis  values=(&min_y to &max_y by &by_y) label= 'Hazard Ratio';
band x= &var
 lower=LowerExp upper=UpperExp / fillattrs=(color=gray transparency=0.6) ;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;
ods graphics / imagename="PH Histogram";

proc sgplot data=&data;
Title "&var";
/*where &min_x le &var le &max_x;*/
histogram &var / scale=count fillattrs=(color=gray transparency=0.6);
run;

ods pdf close;
run;
%mend phregsp;
run;

/* Sample data execution
%phregsp(data=analytic,var=bmi,refval=25,covar=age sex,class=sex,outcome=death,time0=,
           time1=timedth,reftime=365,nknots=8,method=3,weight=NONE,min_x=15,max_x=43,by_x=5,
           min_y=0,max_y=3,by_y=1);
run;
*/


/***********************************************************************************
*                                                                                  *
*   Spline macro for survival outcomes with effect modification/subgroups          *
*                                                                                  *
***********************************************************************************/
%macro phregspem(data,var,refval,emvar,emref,covar,class,outcome,time0,time1,reftime,nknots,
                    method,weight,min_x,max_x,by_x,min_y,max_y,by_y,CI=YES,color=YES);
/*
  data                      Data set name;
  var                       Splined predictor
  refval                    Reference value of spline variable
  emvar                     Stratifying/Effect modifier variable [numeric, coded 1-N]
  emref                     Reference category for effect modifier variable
  covar                     Model covariates
  class                     Class statement variables
  outcome                   Censoring indicator variable [0=censored]
  time0                     Entry time.  If all subjects enter at time=0, leave blank.
  time1                     Exit time.  If all subjects enter at time=0, this is survival time.
  reftime                   Reference time for estimates 
  nknots                    Number of internal knots
  method                    Knot placement method:  1=Even knot placement  
                            2= Percentile based   3= Harrell's approach
  weight                    Weight variable, if needed.  May be blank or "NONE".
  min_x, max_x by_x         minimum, maximum and interval values on X axis of plot
  min_y, max_y by_y         minimum, maximum and interval values of Y axis of plot
  CI                        Do you want confidence intervals?  Default is YES
  color                     Do you want color plots?  Default is YES, otherwise grayscale

  Note that a predicted survival for the reference value at the reference timeis calculated 
  and appears at the end of the output.

*/
%if &color=YES %then %let attrid = colors;
%else %let attrid = grays;

options options ps=1000 validvarname=v7;

/*Calculate Harrell's Knot Placement*/
data FHps;
array pcts(&nknots) pcts1-pcts&nknots;
  expt = ceil(&nknots/3)-1;
  pcts(1) = 0.1/2**expt;
  pcts(&nknots) = 1-pcts(1);
  range = 1-2*pcts(1);
   incr = range/(&nknots-1);
  do i = 2 to &nknots-1;
     pcts(i) = pcts(i-1)+incr;
   end;
  do i = 1 to &nknots;
    pcts(i) = 100*round(pcts(i),.0001);
  end;
   keep pcts1-pcts&nknots;
   proc print data=FHps;
   run;
data _null_;
set FHps;
call symput('FHpcts',catx(' ', of _numeric_));
run;
proc univariate data=&data noprint;
   var &var;
   output out=percentiles pctlpts=&FHpcts pctlpre=p_;run;
data _null_;
set percentiles;
call symput('FHcuts',catx(',', of _numeric_));
run;

/*Prepare data set for spline estimation and graphing*/
proc means data=&data;
var &var ;
output out=meanout min=minx  max=maxx ;
run;

data newdata; set meanout;
interval = (maxx-minx)/100;
        do &var = minx to maxx by interval; 
          output;
        end; 
        keep &var ;
        run;
proc sql noprint;
select &var into :cat1-:cat101 from newdata;
run;
quit;


ods output OneWayFreqs=owf;
proc freq data=&data;
tables &emvar;
run;
proc print data=owf;
run;

data owf; set owf end=eof;
vname = cat('emvar',_n_);
call symput(vname,&emvar);
if eof then call symput("numcats",_n_);
run;
proc print data=owf;
run;
/*Spline modeling:*/

/*Prepare Cumulative Incidence Plot Data at reference value*/
data baseline; set owf;
&var = &refval;
keep &var &emvar;
run;
ods listing;
proc print data=baseline;
run;

proc phreg data=&data ;
title 'Crude Model';
class &emvar;
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
%if &time0 eq %then %do;
model &time1*&outcome.(0) = spl*&emvar;
%end;
%else %do;
model (&time0,&time1)*&outcome.(0) = spl*&emvar;
%end;
baseline out=outtime  survival=survival /diradj group=&emvar;
store preddat2;
run;
quit;
proc sort data=outtime; by &emvar &time1 ;
data outtime; set outtime; by &emvar &time1;
CUMincidence = (1-survival)*100;
data outtime2; set outtime; by &emvar &time1 ;
if &time1 le &reftime;
data outtime2; set outtime2;by &emvar &time1 ;
if last.&emvar;
run;
ods listing;
proc print data=outtime2;
title "Crude Cumulative Incidence at time=&reftime";
run;
proc plm restore=preddat2 ;
title 'Spline knot locations';
show effects;
run;
quit;

/*Run PHREG with spline variables and covariates*/
proc phreg data=&data outest=pecov covout ;
title 'Multivariate adjusted model';
%if &method = 1 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=equal(&nknots)) ; %end;
%if &method = 2 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=percentiles(&nknots)) ; %end;
%if &method = 3 %then %do;
effect spl = spline(&var /naturalcubic knotmethod=list(&FHcuts)) ; %end;
class &class &emvar; 
  %if (&weight ne ) & (&weight ne NONE) %then %do; weight &weight %end;;

%if &time0 eq %then %do;
model &time1*&outcome.(0) = spl &emvar spl*&emvar &covar;
%end;
%else %do;
model (&time0,&time1)*&outcome.(0) = spl &emvar spl*&emvar &covar;
%end;
baseline out=outtimem  survival=survival /diradj group=&emvar;
store preddat;
run;
quit;

proc sort data=outtimem; by &emvar &time1 ;
data outtimem; set outtimem; by &emvar &time1;
CUMincidence = (1-survival)*100;
data outtime2m; set outtimem; by &emvar &time1 ;
if &time1 le &reftime;
data outtime2m; set outtime2m;by &emvar &time1 ;
if last.&emvar;
run;
ods listing;
proc print data=outtime2m;
title "Multivariate Adjusted Cumulative Incidence at time=&reftime";
run;


ods listing close;
ods output Estimates=plmout;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   %do emcat = 1 %to &numcats;
   estimate "&&emvar&emcat &&cat&numcat"  spl [1, &&cat&numcat] [-1, &refval]
            &emvar [1, &&emvar&emcat] [-1 &emref]
            &emvar*spl [1, &&emvar&emcat &&cat&numcat ] [-1, &emref &refval]  /exp cl;
   %end;
%end;
run;
quit;


data plmout2; set plmout;
&emvar = scan(Label,1,' ')/1;
&var = scan(Label,2,' ')/1;
if UpperExp gt &max_y then UpperExp = &max_y;
if LowerExp lt &min_y then LowerExp = &min_y;
run;
/*Range-based comparison*/
ods listing close;
ods output Estimates=tplmout;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   %do emcat = 1 %to &numcats;
   estimate "&&emvar&emcat &&cat&numcat"  spl [1, &&cat&numcat] [-1, &&cat&numcat]
            &emvar [1, &&emvar&emcat] [-1 &emref]
            &emvar*spl [1, &&emvar&emcat &&cat&numcat ] [-1, &emref &&cat&numcat]  /exp cl;
   %end;
%end;
run;
quit;


data tplmout2; set tplmout;
&emvar = scan(Label,1,' ')/1;
&var = scan(Label,2,' ')/1;
if UpperExp gt &max_y then UpperExp = &max_y;
if LowerExp lt &min_y then LowerExp = &min_y;
run;




/*Within EM value contrasts*/

ods listing close;
ods output Estimates=plmoutX;
proc plm restore=preddat ;
%do numcat = 1 %to 101;
   %do emcat = 1 %to &numcats;
   estimate "&&emvar&emcat &&cat&numcat"  spl [1, &&cat&numcat] [-1, &refval]
            &emvar [1, &&emvar&emcat] [-1 &&emvar&emcat]
            &emvar*spl [1, &&emvar&emcat &&cat&numcat ] [-1, &&emvar&emcat &refval]  /exp cl;
   %end;
%end;
run;
quit;


data plmoutX2; set plmoutX;
&emvar = scan(Label,1,' ')/1;
&var = scan(Label,2,' ')/1;
if UpperExp gt &max_y then UpperExp = &max_y;
if LowerExp lt &min_y then LowerExp = &min_y;
run;



/*Graph it*/

options orientation=landscape;
&ods1; &ods2; &ods3;

proc means data=outtime noprint;
var CUMincidence;
output out=CI max=maxCI;
run;
proc means data=outtimem noprint;
var CUMincidence;
output out=CIm max=maxCIm;
run;
data _null_; merge CIm CI;
maxy = max(maxCI,maxCIm);
call symput('maxy',maxy);
run;

ods graphics / imagename="CrudeCumIncidence";

options orientation=landscape;
proc sgplot data=outtime dattrmap=graphattr;
title "Crude Cumulative Incidence Plot";
step x=&time1 y=CUMincidence /group=&emvar attrid=&attrid;
yaxis  label='Cumulative Incidence (%)' max=&maxy;
run;
quit;

options orientation=landscape;
ods graphics / imagename="MVACumIncidence";

proc sgplot data=outtimem dattrmap=graphattr;
title "Multivariate Adjusted Cumulative Incidence Plot";
step x=&time1 y=CUMincidence /group=&emvar attrid=&attrid;
yaxis  label='Cumulative Incidence (%)' max=&maxy;
run;
quit;

/*Standard Hazard Ratio over X plot*/
ods graphics / imagename="STD Hazard";
proc sgplot data=plmout2 noautolegend dattrmap=graphattr;
title ;
series y=ExpEstimate x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  values=(&min_y to &max_y by &by_y) label='Hazard Ratio';
%if &CI ne NO %then %do;
band x= &var
 lower=LowerExp upper=UpperExp / group=&emvar attrid=&attrid ;
%end;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;




/*Standard Hazard Ratio over Log(X) plot*/
ods graphics / imagename="Hazard w/LogX";
proc sgplot data=plmout2 noautolegend dattrmap=graphattr;
title "Log_e(&var)";
series y=ExpEstimate x=&var / group=&emvar attrid=&attrid;
xaxis  TYPE=LOG LOGSTYLE=LOGEXPAND LOGBASE=e values=(&min_x to &max_x by &by_x);

yaxis  values=(&min_y to &max_y by &by_y) label='Hazard Ratio';
%if &CI ne NO %then %do;
band x= &var
 lower=LowerExp upper=UpperExp / group=&emvar attrid=&attrid ;
%end;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;
title;

/*Range of X Hazard Ratio plot*/
ods graphics / imagename="Hazard Range";
proc sgplot data=tplmout2 noautolegend dattrmap=graphattr;
title ;
series y=ExpEstimate x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  values=(&min_y to &max_y by &by_y) label='Hazard Ratio';
%if &CI ne NO %then %do;
band x= &var
 lower=LowerExp upper=UpperExp / group=&emvar attrid=&attrid ;
%end;
refline 1 /axis=y lineattrs=(thickness=2 color=black);
run;
quit;


/*Within EM value contrasts*/
ods graphics / imagename="Hazard w/in EM";
proc sgplot data=plmoutX2 noautolegend dattrmap=graphattr;
title ;
series y=ExpEstimate x=&var / group=&emvar attrid=&attrid;
xaxis  values=(&min_x to &max_x by &by_x)  ;
yaxis  label='Hazard Ratio';
refline 1 /axis=y lineattrs=(thickness=2 color=black);
refline &refval /axis=x lineattrs=(thickness=2 color=black);
run;
quit;



ods graphics / imagename="Histogram";
proc sgpanel data=&data dattrmap=graphattr;
panelby &emvar;
histogram &var / scale=count fillattrs=(color=gray transparency=0.6) ;
run;

ods pdf close;

%mend phregspem;

/*Sample execution:
%phregspem(data=analytic,var=BMI,refval=25,emvar=agegrp3,emref=1,covar=sex,
             class=sex,outcome=death,time0=,time1=timedth,reftime=3653,nknots=5,method=3,
             weight=NONE,min_x=15,max_x=40,by_x=5,min_y=0,max_y=20,by_y=2,CI=YES,color=NO);
run;
*/



